


!-----------------------------------------------------------------------------------------------------------------
!    Copyright (C) 2010  Emanuele Luzio
!    If you are completly lost you can mail me: emanuele.luzio@lngs.infn.it

!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.

!    You should have received a copy of the GNU General Public License
!    along with this program.  If not, see <http://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------------------------------------------
!----------------------------------------------------------------------------------------------------------------
!    Here you can find an example of the gadof subroutine.
!    usage: call gadof(ff,nparameter,x,npopul,nd,nplateau,nmixing)
!    it takes as inputs seven parameters:

!    ff: a fitness function that returns a real*8 number. Must be declared external in the main
 
!    nparameter: the numbers of the parameters of the fitness function ff

!    x: is a vector of dimension nparameter, that will be returned  by gadof subroutine at the end of the job. The range of single element is [0,1], so be carefull
!    in the rescaling of your fitness function.

!    npopul: the dimension of the population. 20 or 30 is a quite good dimension

!    nd: the number of significative digits that you would like use. small=fast but poor resolution, big=slow but high resolution. 3~4 is a good dimension

!    nplateu: the number of iteration after that you will be satisfied in accuracy

!    nmixing: is the number of random vectors to start investigating the pattern space. small=faster but poor resolution, big=slower but high resolution.


!-----------------------------------------------------------------------------------------------------------------



        program example
        parameter (nparam=4)
        real vec(nparam)
	external fitfunc
	
!	read your data on some external file to pass them to fitnessfunction


!	reminder of usage: 
!       call gadof(ff,nparameter,x,npopul,nd,nplateau,nmixing)
      	call gadof(fitfunc,nparam,vec,20,3,100000,1000)

!	use your vec

	stop 
	end



        real*8 function fitfunc(x)
	parameter (n=4)
	real x(n)
	!evaluate something
	fitfunc=dble(x(1)*x(3)-x(3)*x(4)+sqrt(x(2)))!<------This is only a no means example ^_^
        return
        end




	subroutine gadof(ff,nparameter,x,npopul,nd,nplateau,nmixing)
	integer nplateau,npopul,nd
	integer indx,nparameter,i,j,q,k
	integer gval,cycl
	real*8 fit,fitmed,ff,ratemax
	real seed,x(nparameter)
	real parents(nparameter,npopul),ymen(nparameter),xmen(nparameter),rparents(nparameter,npopul)
	real*8 rate(npopul),f(npopul),rndm48(nplateau)
	integer codedparents(nparameter*nd,npopul),codedofs(nparameter*nd,npopul)
	external ff


!-------Intialization----------------------
!------------------------------------------
	write(*,*)
	write(*,*)'================================================='	
	write(*,*)'          Genetic Algorithm - v04.2010           '
	write(*,*)'================================================='
	call rm48(rndm48,nplateau)
	write(*,*)'================================================='
	write(*,*)

	ratemax=0.d0
	
	do nmix=1,nmixing
	cycl=0

	do i=1,npopul
		do j=1,nparameter
			seed=real(j)
			parents(j,i)=rndm(seed)
		end do
	end do

!------------------------------------------
!------------------------------------------



	do  while(cycl < nplateau)

!-------Fittness--------------------------------------------------------------
!-------If a better fit pattern is found, then is wrote in the GA.out file----
!-------to allow  interactive investigation of the pattern space. And is wrote
!--------to a standart output to the progress of subroutine-------------------
	fit=0.d0

	do i=1,npopul
		do k=1,nparameter
		ymen(k)=parents(k,i)
		end do
		rate(i)=ff(ymen)
		if (rate(i).gt.ratemax) then
		ratemax=rate(i)
		write(*,"(1x,A57,D12.4,A27,f6.2,A1)",advance="no") &
		'\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b ff: ', & 
		ratemax,'             Progress:',100.*real(nmix)/real(nmixing),'%'
		cycl=0
		do k=1,nparameter
		xmen(k)=parents(k,i)
		end do
		open(10,file='GA.out',status='unknown')
		do k=1,nparameter
		write(10,"(1x,f8.3,5x,i2)")xmen(k),k
		end do
		close (10)


		end if

	
		cycl=cycl+1	!<----index that will be evaluated in the Plateau condition	
		fit=fit+rate(i)

	end do

	fitmed=fit/dble(npopul)

	do i=1,npopul
		rate(i)=rate(i)/fitmed
	end do

!-------Roulette-------------------------------------------------------
!-------A double precision roulette based on mathlib- of cernlib-------	
	do i=1,npopul
		f(i)=0.d0
	end do


	do i=1,npopul
		do j=1,i
		f(i)=f(i)+rate(j)
		end do
	end do
	do j=1,npopul

	seed=f(npopul)*rndm48(cycl)	
	do i=1,npopul
		if(seed < f(npopul + 1-i)) indx=npopul + 1-i
	end do
	do k=1,nparameter
	rparents(k,j)=parents(k,indx)
	end do

	end do

!---------Codes------------------------------------
!---------A base 10 coding of the input parameters-
	do k=1,npopul
	do i=1,nparameter
		do j=1,nd
			q=nd-j+1
			codedparents(nd*(i-1)+j,k)=int(mod(10.**real(q)*rparents(i,k),10.))
		end do
	end do
	end do


!--------Mutes-------------------------------
!--------this is a single option of mutation-
!--------This on fo the most important parts- 
!--------in order to improve velocity--------
!--------Please help mostly in this part-----

	do k=1,npopul
	indx=int(rndm(seed)*real(nparameter*nd)+ 1.)

	gval=int(10.*rndm(seed))
	codedparents(indx,k)=gval

	end do


!--------Cross--------------------------------------------
!--------This is a single point implementation of crossing
!---------------------------------------------------------

	do k=1,npopul-1,2
		indx=nd*nparameter/2
		do i=1,indx
			codedofs(i,k)=codedparents(i,k)
			codedofs(i,k+1)=codedparents(i,k+1)
		end do
		do i=indx,nd*nparameter
			codedofs(i,k+1)=codedparents(i,k)
			codedofs(i,k)=codedparents(i,k+1)
		end do
	end do

!--------Decodes--------------------------
!--------Of course the opposite of coding-
	do i=1,npopul
		do j=1,nparameter
			parents(j,i)=0.
		end do
	end do

	do i=1,npopul
		do j=1,nparameter
			do k=1,nd
			parents(j,i)=parents(j,i)+ real(codedofs(nd*(j-1)+k,i)*10.**real(k))
			end do
			parents(j,i)=parents(j,i)*0.1**real(nd+1)	
		end do
	end do
	end do
		write(*,"(1x,A57,D12.4,A27,f6.2,A1)",advance="no") &
		'\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b ff: ', & 
		ratemax,'             Progress:',100.*real(nmix)/real(nmixing),'%'
	
	end do

	do i=1,nparameter
	x(i)=xmen(i)
	end do
	write(*,*)

	end



